import "package:carousel_slider/carousel_slider.dart";
import 'package:flutter/services.dart';
import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/files/image_viewer_storage.dart';
import 'package:verttice_ui/src/widgets/files/pdf_viewer_storage.dart';
import 'package:verttice_ui/src/widgets/files/video_viewer_storage.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class FileListViewStorage extends StatefulWidget {
  final List<FileModel> files;
  final bool showAsGrid;
  final double? width;
  final double? height;
  final Function(FileModel imageToDelete)? deleteImageCallback;
  const FileListViewStorage({
    Key? key,
    required this.files,
    this.deleteImageCallback,
    this.height,
    this.width,
    this.showAsGrid = false,
  }) : super(key: key);

  @override
  State createState() => _FileListViewState();
}

class _FileListViewState extends State<FileListViewStorage> {
  int _current = 0;
  final CarouselController _controller = CarouselController();
  List<FileModel> _files = [];
  @override
  void initState() {
    super.initState();
    _files = widget.files;
  }

  @override
  void didUpdateWidget(covariant FileListViewStorage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_files != widget.files) {
      _files = widget.files;
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(builder: (context, constraints) {
      final width = widget.width ??
          (constraints.maxWidth != double.infinity
              ? constraints.maxWidth
              : MediaQuery.of(context).size.width);

      final height = widget.height ??
          (constraints.maxHeight != double.infinity
              ? constraints.maxHeight
              : 250);
      return SizedBox(
        height: height,
        width: width,
        child: Column(children: [
          Expanded(
            child: widget.showAsGrid
                ? Wrap(
                    runAlignment: WrapAlignment.spaceAround,
                    alignment: WrapAlignment.spaceAround,
                    runSpacing: 8,
                    spacing: 5,
                    direction: Axis.vertical,
                    children: _files
                        .map((e) => _FileItem(
                              height: 100,
                              width: 100,
                              file: e,
                              deleteImageCallback: widget.deleteImageCallback,
                            ))
                        .toList(),
                  )
                : CarouselSlider(
                    items: _files
                        .map((e) => _FileItem(
                              height: height,
                              width: width * 0.7,
                              file: e,
                              deleteImageCallback: widget.deleteImageCallback,
                            ))
                        .toList(),
                    carouselController: _controller,
                    options: CarouselOptions(
                      autoPlay: true,
                      enlargeCenterPage: true,
                      height: 230,
                      onPageChanged: (index, reason) {
                        setState(() {
                          _current = index;
                        });
                      },
                    ),
                  ),
          ),
          if (!widget.showAsGrid)
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: _files.asMap().entries.map((entry) {
                return GestureDetector(
                  onTap: () => _controller.animateToPage(entry.key),
                  child: Container(
                    width: 12.0,
                    height: 12.0,
                    margin: const EdgeInsets.symmetric(
                        vertical: 8.0, horizontal: 4.0),
                    decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        color: (Theme.of(context).brightness == Brightness.dark
                                ? Colors.white
                                : Colors.black)
                            .withOpacity(_current == entry.key ? 0.9 : 0.4)),
                  ),
                );
              }).toList(),
            ),
        ]),
      );
    });
  }
}

class _FileItem extends StatelessWidget {
  final double height;
  final double width;
  final FileModel file;
  final Function(FileModel imageToDelete)? deleteImageCallback;

  const _FileItem({
    Key? key,
    required this.height,
    required this.width,
    required this.file,
    this.deleteImageCallback,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        SizedBox(
          width: 50,
          child: Column(children: [
            Tooltip(
              message: 'Copy file link',
              child: CircleAvatar(
                backgroundColor: Colors.blue,
                child: IconButton(
                  onPressed: () async {
                    await Clipboard.setData(
                      ClipboardData(
                        text: file.url,
                      ),
                    );
                    ThemedToast.info(
                      context: context,
                      message: "File LINK copied to clipboard",
                    );
                  },
                  icon: const Icon(
                    Icons.copy,
                    color: Colors.white,
                  ),
                ),
              ),
            ),
            if (deleteImageCallback != null)
              Tooltip(
                message: 'Delete file',
                child: CircleAvatar(
                  backgroundColor: Colors.red,
                  child: IconButton(
                    icon: const Icon(
                      Icons.delete_outlined,
                      color: Colors.white,
                    ),
                    onPressed: () {
                      showDialog(
                        context: context,
                        builder: (BuildContext context) {
                          return AlertDialog(
                            title: const SelectableText("Delete file"),
                            content: const SelectableText(
                                "Are you sure you want to delete this file?"),
                            actions: [
                              TextButton(
                                child: const Text("OK"),
                                onPressed: () {
                                  if (deleteImageCallback != null) {
                                    deleteImageCallback!(file);
                                  }
                                  Navigator.of(context).pop();
                                },
                              ),
                              TextButton(
                                child: const Text("Cancel"),
                                onPressed: () {
                                  Navigator.of(context).pop();
                                },
                              ),
                            ],
                          );
                        },
                      );
                    },
                  ),
                ),
              ),
          ]),
        ),
        Expanded(
          child: Container(
            decoration: const BoxDecoration(
              color: Colors.lightBlue,
              borderRadius: BorderRadius.all(Radius.circular(5.0)),
            ),
            height: height,
            width: width,
            child: Builder(
              builder: (BuildContext context) {
                if (file.mimeType.contains("image")) {
                  return ImageViewerStorage(
                    path: file.path,
                    height: height,
                    width: width,
                  );
                } else if (file.mimeType.contains("pdf")) {
                  return PdfViewerStorage(
                    path: file.path,
                    height: height,
                  );
                } else if (file.mimeType.contains("video")) {
                  return VideoViewerStorage(
                    path: file.path,
                    height: height,
                  );
                }
                return Center(
                  child: ThemedIcons.fileUnknow(
                    iconStyle: IconStyle(
                      color: Colors.amber[600],
                      size: 45,
                    ),
                  ),
                );
              },
            ),
          ),
        ),
      ],
    );
  }
}
