import "dart:typed_data";
import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import "package:flutter/material.dart";
import "package:photo_view/photo_view.dart";
import 'package:verttice_ui/src/widgets/download_button.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_progress.dart';

class ImageViewer extends StatefulWidget {
  final String? url;
  final Uint8List? data;
  final String? asset;
  final double? height;
  ImageViewer({
    Key? key,
    this.url,
    this.data,
    this.asset,
    this.height,
  }) : super(key: key) {
    assert(
      url != null || data != null || asset != null,
      "Must provide url or data or asset",
    );
  }
  @override
  State createState() => _ImageViewerState();
}

class _ImageViewerState extends State<ImageViewer> {
  late String? _url;
  late Uint8List? _data;
  late String? _asset;
  late String _imageID;

  @override
  void initState() {
    super.initState();
    _url = widget.url;
    _data = widget.data;
    _asset = widget.asset;
    _imageID = IDs.l18();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ImageViewer oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_url != widget.url ||
        !const DeepCollectionEquality().equals(_data, widget.data) ||
        _asset != widget.asset) {
      _url = widget.url;
      _data = widget.data;
      _asset = widget.asset;
      _imageID = IDs.l18();
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.height ?? MediaQuery.of(context).size.height / 2.5,
      child: Material(
        child: InkWell(
          child: Hero(
            tag: _imageID,
            child: Center(
              child: Container(
                decoration: BoxDecoration(
                  color: Colors.blueGrey[700]!.withOpacity(0.5),
                  borderRadius: const BorderRadius.all(
                    Radius.circular(7.0),
                  ),
                ),
                child: Builder(builder: (context) {
                  if (_url != null) {
                    return ClipRRect(
                      borderRadius: BorderRadius.circular(7.0),
                      clipBehavior: Clip.hardEdge,
                      child: Image.network(
                        _url!,
                        fit: BoxFit.cover,
                        errorBuilder: (
                          BuildContext context,
                          Object object,
                          StackTrace? trace,
                        ) {
                          return SizedBox(
                            child: Center(
                              child: ThemedIcons.fileUnknow(
                                iconStyle: IconStyle(
                                  size: 17,
                                ),
                              ),
                            ),
                          );
                        },
                        loadingBuilder: (
                          BuildContext context,
                          Widget child,
                          ImageChunkEvent? loadingProgress,
                        ) {
                          if (loadingProgress == null) {
                            return SizedBox(
                              child: ClipRRect(
                                clipBehavior: Clip.hardEdge,
                                borderRadius: BorderRadius.circular(7.0),
                                child: child,
                              ),
                            );
                          }
                          return SizedBox(
                            child: Center(
                              child: ThemedProgress(
                                progress: loadingProgress.expectedTotalBytes !=
                                        null
                                    ? loadingProgress.cumulativeBytesLoaded /
                                        loadingProgress.expectedTotalBytes!
                                    : null,
                              ),
                            ),
                          );
                        },
                      ),
                    );
                  } else if (_data != null) {
                    return ClipRRect(
                      borderRadius: BorderRadius.circular(7.0),
                      child: Image.memory(
                        _data!,
                        fit: BoxFit.cover,
                        errorBuilder: (
                          BuildContext context,
                          Object object,
                          StackTrace? trace,
                        ) {
                          return SizedBox(
                            child: Center(
                              child: ThemedIcons.fileUnknow(
                                iconStyle: IconStyle(
                                  size: 17,
                                ),
                              ),
                            ),
                          );
                        },
                      ),
                    );
                  }
                  return ClipRRect(
                    borderRadius: BorderRadius.circular(7.0),
                    child: Image.asset(
                      _asset!,
                      fit: BoxFit.cover,
                      errorBuilder: (
                        BuildContext context,
                        Object object,
                        StackTrace? trace,
                      ) {
                        return SizedBox(
                          child: Center(
                            child: ThemedIcons.fileUnknow(
                              iconStyle: IconStyle(
                                size: 17,
                              ),
                            ),
                          ),
                        );
                      },
                    ),
                  );
                }),
              ),
            ),
          ),
          onTap: () {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: ((context) => ImageViewerFullPage(
                      imageID: _imageID,
                      asset: _asset,
                      data: _data,
                      url: _url,
                    )),
              ),
            );
          },
        ),
      ),
    );
  }
}

class ImageViewerFullPage extends StatefulWidget {
  final String? url;
  final String? asset;
  final Uint8List? data;
  final String imageID;
  const ImageViewerFullPage({
    Key? key,
    this.url,
    this.asset,
    this.data,
    required this.imageID,
  }) : super(key: key);

  @override
  ImageViewerFullPageState createState() => ImageViewerFullPageState();
}

class ImageViewerFullPageState extends State<ImageViewerFullPage> {
  int _quarterTurns = 0;
  @override
  Widget build(BuildContext context) {
    ImageProvider<Object> provider;
    late DownloadButton downloadWidget;
    if (widget.url != null) {
      provider = NetworkImage(widget.url!);
      downloadWidget = DownloadButton(
        url: widget.url!,
        fileType: VFileType.png,
      );
    } else if (widget.data != null) {
      provider = MemoryImage(widget.data!);
      downloadWidget = DownloadButton(
        fileData: widget.data!,
        fileType: VFileType.png,
      );
    } else {
      provider = AssetImage(widget.asset!);
      downloadWidget = DownloadButton(
        asset: widget.asset!,
        fileType: VFileType.png,
      );
    }
    return Scaffold(
      appBar: ThemedAppBar(
        actions: [
          sizedBoxW8,
          if (widget.url != null)
            CircleAvatar(
              backgroundColor: Colors.lightBlue,
              child: downloadWidget,
            ),
          sizedBoxW8,
          CircleAvatar(
            backgroundColor: Colors.lightBlue,
            child: IconButton(
              onPressed: () {
                setState(() {
                  _quarterTurns =
                      _quarterTurns + 1 == 4 ? 0 : _quarterTurns + 1;
                });
              },
              icon: ThemedIcons.refresh(),
            ),
          ),
          sizedBoxW8,
        ],
      ),
      body: Container(
        color: Colors.blueGrey[700]!.withOpacity(0.5),
        child: RotatedBox(
          quarterTurns: _quarterTurns,
          child: PhotoView(
            onTapDown: (
              BuildContext context,
              TapDownDetails details,
              PhotoViewControllerValue controller,
            ) {
              Navigator.pop(context);
            },
            backgroundDecoration: const BoxDecoration(
              color: Colors.transparent,
            ),
            imageProvider: provider,
            heroAttributes: PhotoViewHeroAttributes(tag: widget.imageID),
            initialScale: PhotoViewComputedScale.contained,
          ),
        ),
      ),
    );
  }
}
