import 'dart:async';
import 'package:collection/collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_quill/flutter_quill.dart' hide Text;
import 'package:pointer_interceptor/pointer_interceptor.dart';
import 'package:provider/provider.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/verttice_ui.dart';

import 'editor_universal_ui/universal_ui.dart';
import 'files_manager.dart';

class ThemedEditor extends StatefulWidget {
  final List<dynamic>? jsonContect;
  final List<FileModel>? files;
  final String? pathToStoreMedia;
  final bool readonly;
  final Future<void> Function({required List<dynamic> jsonContect})?
      fileContentCallback;
  final Future<void> Function({required List<FileModel> files})?
      filesDataCallback;
  const ThemedEditor({
    Key? key,
    this.jsonContect,
    this.files,
    this.readonly = false,
    this.fileContentCallback,
    this.pathToStoreMedia,
    this.filesDataCallback,
  }) : super(key: key);

  @override
  _EditorState createState() => _EditorState();
}

class _EditorState extends State<ThemedEditor> {
  List<dynamic>? _jsonContect;
  QuillController? _controller;
  List<FileModel>? _files;
  final FocusNode _focusNode = FocusNode();
  bool _isPreviewMode = false;

  @override
  void initState() {
    super.initState();
    _files = widget.files;
    _jsonContect = widget.jsonContect;
    if (_jsonContect != null && _jsonContect!.isNotEmpty) {
      _controller = QuillController(
        document: Document.fromJson(_jsonContect!),
        selection: const TextSelection.collapsed(offset: 0),
      );
    } else {
      _controller = QuillController.basic();
    }
  }

  @override
  void didUpdateWidget(covariant ThemedEditor oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_files, widget.files)) {
      _files = widget.files;
    }
    if (!const DeepCollectionEquality()
        .equals(_jsonContect, widget.jsonContect)) {
      _jsonContect = widget.jsonContect;
      if (_jsonContect != null && _jsonContect!.isNotEmpty) {
        _controller = QuillController(
          document: Document.fromJson(_jsonContect!),
          selection: const TextSelection.collapsed(offset: 0),
        );
      } else {
        _controller = QuillController.basic();
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    if (_controller == null) {
      return const Scaffold(body: Center(child: Text('Loading...')));
    }

    return Consumer<VertticeUIState>(
        builder: (BuildContext contxt, VertticeUIState value, Widget? _) {
      final notReadonly = (!widget.readonly && !_isPreviewMode);
      var quillEditor = QuillEditor(
        showCursor: notReadonly,
        controller: _controller!,
        scrollController: ScrollController(),
        scrollable: notReadonly,
        focusNode: _focusNode,
        autoFocus: false,
        readOnly: (widget.readonly || _isPreviewMode),
        placeholder:
            (widget.readonly || _isPreviewMode) ? null : 'Write here...',
        expands: false,
        enableInteractiveSelection: notReadonly,
        padding: EdgeInsets.zero,
      );
      if (kIsWeb) {
        quillEditor = QuillEditor(
          showCursor: notReadonly,
          controller: _controller!,
          scrollController: ScrollController(),
          scrollable: notReadonly,
          focusNode: _focusNode,
          autoFocus: false,
          readOnly: (widget.readonly || _isPreviewMode),
          placeholder:
              (widget.readonly || _isPreviewMode) ? null : 'Write here...',
          expands: false,
          enableInteractiveSelection: notReadonly,
          padding: const EdgeInsets.all(4),
          embedBuilder: defaultEmbedBuilderWeb,
        );
      }
      var toolbar = QuillToolbar.basic(
        controller: _controller!,
        showAlignmentButtons: true,
        showImageButton: true,
        showVideoButton: true,
        multiRowsDisplay: !ScreenSizeUtils.isMobile(context: context),
      );
      if (widget.readonly) {
        return Container(
          padding: const EdgeInsets.all(16),
          child: quillEditor,
        );
      }
      return PointerInterceptor(
        child: Scaffold(
          floatingActionButton: widget.fileContentCallback == null
              ? null
              : FloatingActionButton(
                  onPressed: () async {
                    if (_controller != null) {
                      widget.fileContentCallback!(
                        jsonContect: _controller!.document.toDelta().toJson(),
                      );
                    }
                  },
                  child: ThemedIcons.check(),
                ),
          body: SafeArea(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Flex(
                  direction: ScreenSizeUtils.isMobile(context: context)
                      ? Axis.vertical
                      : Axis.horizontal,
                  children: [
                    Row(
                      children: [
                        if (widget.filesDataCallback != null && !_isPreviewMode)
                          Padding(
                            padding: const EdgeInsets.all(15.0),
                            child: Tooltip(
                              message:
                                  'Files manager for editor, you can upload files here and get LINK to put in the editor.',
                              child: CircleAvatar(
                                child: IconButton(
                                  onPressed: () async {
                                    await fullPageDialog(
                                      context: context,
                                      builder: (BuildContext context) {
                                        return EditorFileMangerView(
                                          files: _files ?? [],
                                          path: widget.pathToStoreMedia ??
                                              "editorFiles",
                                          filesDataCallback: ({
                                            required List<FileModel> files,
                                          }) async {
                                            await widget.filesDataCallback!(
                                              files: files,
                                            );
                                          },
                                        );
                                      },
                                    );
                                  },
                                  icon: ThemedIcons.files(),
                                ),
                              ),
                            ),
                          ),
                        Padding(
                          padding: const EdgeInsets.all(15.0),
                          child: Tooltip(
                            message: 'Show preview of the article',
                            child: Switch(
                              value: _isPreviewMode,
                              onChanged: (bool on) {
                                setState(() {
                                  _isPreviewMode = !_isPreviewMode;
                                });
                              },
                            ),
                          ),
                        ),
                      ],
                    ),
                    if (!_isPreviewMode)
                      if (ScreenSizeUtils.isMobile(context: context))
                        Container(
                          padding: !kIsWeb
                              ? null
                              : const EdgeInsets.symmetric(
                                  vertical: 16,
                                  horizontal: 8,
                                ),
                          child: toolbar,
                        )
                      else
                        Expanded(
                          child: Container(
                            padding: !kIsWeb
                                ? null
                                : const EdgeInsets.symmetric(
                                    vertical: 16,
                                    horizontal: 8,
                                  ),
                            child: toolbar,
                          ),
                        )
                  ],
                ),
                const SizedBox(
                  height: 8,
                ),
                Expanded(
                  flex: 15,
                  child: Container(
                    padding: const EdgeInsets.only(left: 16, right: 16),
                    child: quillEditor,
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    });
  }
}
